scala 2.11 で確認

// 普通
object Hello {
  def main(args: Array[String]) {
    println("Hello, World")
  }
}

// Appをつかった普通
object HelloExtendsApp extends App {
  println("Hello, World")
}

// App をつかって mainをオーバーライド
// 入門できない
// Appでmainで実装されているため
// App内部の initCode に保存されるがmainで呼び出されているため
object HelloExtendsAppOverrideMain extends App {
  println("hello, World")

  override def main(args: Array[String]) = println("start main")
}

// App をつかってして delayedInitをオーバーライド
// 入門できない
// App内部の initCode には保存されないためmainで呼び出すものがない
object HelloExtendsAppOverrideDelayedInit extends App {
  println("hello, World")

  override def delayedInit(body: => Unit) = println("start delayedInit")
}


// 参考
//
// delayedInitが先
object HelloExtendsAppOverrideBoth extends App {
  println("hello, World")

  override def delayedInit(body: => Unit) = println("start delayedInit")
  override def main(args: Array[String]) = println("start main")
}

// ずるいやつ
// 入門できる
object HelloBad {
  println("Hello, World")

  def main(args: Array[String]) = println("start main")
}


// DelayedInit
// 入門できない
// bodyに渡るが呼び出しをしている人がいない
object HelloExtendsDelayedInit extends DelayedInit{
  println("Hello, World")

  override def delayedInit(body: => Unit) = println("start delayedInit")
  def main(args: Array[String]) = println("start main")
}

// 入門できる
// 自分で呼べばいい
object HelloExtendsDelayedInitCallBody extends DelayedInit{
  println("Hello, World")

  override def delayedInit(body: => Unit) = body
  def main(args: Array[String]) = println("start main")
}

// 仕組み コンパイル時に DelayedInitのサブクラスであれば特殊な処理がされる
//
// delayedEndpoint
// https://github.com/scala/scala/blob/540b18fb68a0210b187e595622c31f20b2c6f581/src/compiler/scala/tools/nsc/transform/Constructors.scala#L474
// まとめ DelayedInitを使うと
    static
    {
        new HelloBad$();
    }
で呼ばれるコードをコンパイル時に分離して static {} 内で呼ばれなくして変わりに delayedInitを呼ぶ
このとき本来呼ばれていたコードを引数bodyでうけとる

DelayedInit tait の実装だけみてたら気がつかない…からい
https://github.com/scala/scala/blob/v2.11.6/src/library/scala/DelayedInit.scala#L1

こういった特殊なものがあるならもっとしりたい
